function [handle,w3d] = plot_3Dobj(x,y,z,roll,pitch,yaw,w3d,handle);
%[handle,w3d] = plot_robot(x, y, z, roll, pitch, yaw, w3d)
%Creates a figure representing the model contained in the w3d variable. As
%a result of this initialization, the w3d model is altered and should be
%updated outside the function. Handle is the handle of the model plot and
%is necessary to redraw it.

%plot_robot(x, y, z, roll, pitch, yaw, w3d, handle)
%actualizes the model plot with a new position x,y,z,roll,pitch,yaw and
%knowing the handle variable from a previously created model.

%To create a new model you need to generate it in vrml 1.0 and then use the
%read_vrml.m matlab function to obtain the w3d variable. To have everything
%in a unique container I store the "nel" variable into "w3d(1).nel".


M=mrot(roll,pitch,yaw);

if nargin==7
    hold on
    for i=1:w3d(1).nel %for each object on the model
        
        %Create the 3d surface plot
        handle(i)=trisurf(w3d(i).knx,w3d(i).pts(:,1),w3d(i).pts(:,2),w3d(i).pts(:,3),'FaceColor',w3d(i).color,'edgecolor','none');
        %The secuence of points is altered according to connectivity. Here
        %the new set of points is recovered and stored again in w3d to use
        %the same sequence in succesive calls
        a=get(handle(i),'XData');
        b=get(handle(i),'YData');
        c=get(handle(i),'ZData');
        w3d(i).pts=[a(:) b(:) c(:)];

        %Do an update using the new stored points and the x, y, z, roll, 
        %pitch and yaw coordinates
        drw(i).pts=(M*w3d(i).pts')';
        drw(i).pts(:,1)=drw(i).pts(:,1)+x;
        drw(i).pts(:,2)=drw(i).pts(:,2)+y;
        drw(i).pts(:,3)=drw(i).pts(:,3)+z;
        l=length(drw(i).pts(:,1))/3;
        a=reshape(drw(i).pts(:,1),3,l);
        b=reshape(drw(i).pts(:,2),3,l);
        c=reshape(drw(i).pts(:,3),3,l);
        set(handle(i),'XData',a,'YData',b,'ZData',c,'FaceColor',w3d(i).color);
    end
elseif nargin==8
    for i=1:w3d(1).nel
        drw(i).pts=(M*w3d(i).pts')';
        drw(i).pts(:,1)=drw(i).pts(:,1)+x;
        drw(i).pts(:,2)=drw(i).pts(:,2)+y;
        drw(i).pts(:,3)=drw(i).pts(:,3)+z;
        l=length(drw(i).pts(:,1))/3;
        a=reshape(drw(i).pts(:,1),3,l);
        b=reshape(drw(i).pts(:,2),3,l);
        c=reshape(drw(i).pts(:,3),3,l);
        set(handle(i),'XData',a,'YData',b,'ZData',c,'FaceColor',w3d(i).color);
    end
end





