%Entrades: Qpos (valor dels angles), Q_dot (Velocitat de variaci dels angles), Q_2dot (acceleraci de variaci dels angles), F_ext (fora exterior) i Grv %(Gravetat)
%function tau=RNE(Qpos,Q_dot,Q_2dot,F_ext,Grv);
function FNBase = rneQLM(Qpos, Q_dot, Q_2dot, fext, Grv, Lon, Massa)

%Sinicialitza tot ha 0 excepte lacceleraci linial que ha de tenir en compte el valor de 
%la gravetat
velocitatAngular = [0;0;0];
acceleracioAngular = [0;0;0];
velocitatLineal = [0;0;0];
acceleracioLineal = Grv';

%Z s un vector de 3 posicions constant.
Z = [0;0;1];

%Forces i moments inicialment buits
F=[];
N=[];


%Longituds dels diferents links.
p{1}=[Lon(1);0;0];
p{2}=[Lon(3);0;0];
p{3}=[Lon(4);0;0];
p{4}=[0;0;0];
p{5}=[0;0;Lon(5)];

%Masses de les diferents unions (links)
mArm=Massa;
m=mArm;

%Inercia dels cossos
I{1}=m(1)/12*diag([2*((1+0.1^2)*Lon(2))^2;(1+0.1^2)*Lon(2)^2;(0.1)*Lon(2)^2]);
I{2}=m(2)/12*diag([2*(0.1*Lon(3))^2;(1+0.1^2)*Lon(3)^2;(1+0.1^2)*Lon(3)^2]);
I{3}=m(3)/12*diag([2*(0.1*Lon(4))^2;(1+0.1^2)*Lon(4)^2;(1+0.1^2)*Lon(4)^2]);
I{4}=m(4)/12*diag([2*(0.1*0)^2;(1+0.1^2)*0^2;(1+0.1^2)*0^2]);
I{5}=m(5)/12*diag([2*((1+0.1^2)*Lon(5))^2;(1+0.1^2)*Lon(5)^2;(0.1)*Lon(5)^2]);

%I{1} = diag([1.1 1.11 0.177]);
%I{2} = diag([0.130 0.524 0.539]);
%I{3} = diag([66e-3 12.5e-3 86e-3]);
%I{4} = diag([300e-6 300e-6 400e-6]);
%I{5} = diag([150e-6 150e-6 40e-6]);

%Sobtenen les matrius de rotacions que seran necessries per resoldre les equacions 
%del clcul de la dinmica inversa.
R = rotacions(Qpos);

%Per cada articulaci es calcula la fora i el moment
%Totes aquestes frmules sn idntiques a les de la secci terica, per tant no es 
%tornaran a comentar amb detall.
for i=1:5
    
    %inicialment es calcula lacceleraci angular
    acceleracioAngular = R{i}*acceleracioAngular+(cross(R{i}*velocitatAngular, R{i}*Q_dot(i)*Z)+R{i}*Q_2dot(i)*Z);

    %Es calcula la nova velocitat angular
    velocitatAngular = R{i}*velocitatAngular+ R{i}*Q_dot(i)*Z;

   %Es calcula lacceleraci lineal en el centre de masses
    acceleracioLinealC = R{i} * acceleracioLineal + cross(velocitatAngular, cross(velocitatAngular, p{i}/2)) + cross(acceleracioAngular, p{i}/2);
    
   %Es calcula la nova acceleraci lineal
    acceleracioLineal = R{i} * acceleracioLineal + cross(velocitatAngular, cross(velocitatAngular, p{i})) + cross(acceleracioAngular, p{i});
    
    %Es troba la fora relacionada amb larticulaci actual, fent massa per acceleraci
    Fm = m(i)*acceleracioLinealC;

    %Es calcula el moment tamb de la articulaci actual
    Nm = I{i}*acceleracioAngular+cross(velocitatAngular,I{i}*velocitatAngular);

    %Es van guardant les forces i el moments de les diferents articulacions en una 
    %matriu
    F=[F Fm];
    N=[N Nm];
end

%Safegeixen les forces externes que semergeixen sobre el manipulador
fn1 = fext(1:3);
nn1 = fext(4:6);

f = fn1 + F(:,5);
n = nn1 + cross(p{5}, fn1) + cross((p{5}/2),F(:,5))+N(:,5);

%Es troba la tau referent a lultima articulaci
tau(1,5) = (n)' * (R{5}*Z);

%Es troben la resta de taus
for i=4:-1:1
    n = (R{i+1})'*(n+cross((R{i+1})*p{i},f))+cross(p{i}/2, F(:,i))+N(:,i);
    f= (R{i+1})'*f+F(:,i);
    tau(1,i)= (n)'*(R{i}*Z);     
end

%Es retorna la tau tranposada
FNBase = (tau)';